"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const jsYaml = require("js-yaml");
const loadYamlFile = require("load-yaml-file");
const path = require("path");
const base_1 = require("../base");
const labels_1 = require("../schemas/labels");
const settings_1 = require("../schemas/settings");
const esClient_1 = require("../utils/es/esClient");
const index_1 = require("../utils/github/fetchLabels/index");
const chunkArray_1 = require("../utils/misc/chunkArray");
class GhLabels extends base_1.default {
    /*
      The aim of this script is to fetch all labels associated with active repositories.
      It does the following:
       - Fetch a list of repositories from Elasticsearch
       - Fetch updated labels for each repository
       - Send back the content to Elasticsearch
     */
    async run() {
        const { flags } = this.parse(GhLabels);
        const userConfig = await loadYamlFile(path.join(this.config.configDir, 'config.yml'));
        const { esnode, esca, escloudid, escloudusername, escloudpassword, esrepo, gtoken, gincrement } = flags;
        const es_node = (esnode !== undefined ? esnode : userConfig.elasticsearch.node);
        const es_ssl_ca = (esca !== undefined ? esca : userConfig.elasticsearch.sslca);
        const es_cloud_id = (escloudid !== undefined ? escloudid : userConfig.elasticsearch.cloud.id);
        const es_cloud_username = (escloudusername !== undefined ? escloudusername : userConfig.elasticsearch.cloud.username);
        const es_cloud_password = (escloudpassword !== undefined ? escloudpassword : userConfig.elasticsearch.cloud.password);
        const reposIndexName = (esrepo !== undefined ? esrepo : userConfig.elasticsearch.indices.repos);
        const indexLabelPrefix = userConfig.elasticsearch.indices.labels;
        const gh_token = (gtoken !== undefined ? gtoken : userConfig.github.token);
        const gh_increment = parseInt((gincrement !== undefined ? gincrement : userConfig.fetch.max_nodes), 10);
        //1- Test if an index exists, if it does not, create it.
        cli_ux_1.default.action.start('Checking if index: ' + reposIndexName + ' exists');
        const client = await esClient_1.default({
            es_node,
            es_ssl_ca,
            es_cloud_id,
            es_cloud_username,
            es_cloud_password,
        });
        const healthCheck = await client.cluster.health();
        if (healthCheck.body.status === 'red') {
            this.log('Elasticsearch cluster is not in an healthy state, exiting');
            this.log(healthCheck.body);
            process.exit(1);
        }
        const testIndex = await client.indices.exists({ index: reposIndexName });
        if (testIndex.body === false) {
            this.error('Index: ' + reposIndexName + ' does not exists, please configure repositories first', { exit: 1 });
        }
        cli_ux_1.default.action.stop(' done');
        //2- Grab the active repositories from Elasticsearch
        cli_ux_1.default.action.start('Grabbing the active repositories from ElasticSearch');
        let esRepos = await client.search({
            index: reposIndexName,
            body: {
                from: 0,
                size: 10000,
                query: {
                    match: {
                        active: {
                            query: true
                        }
                    }
                }
            }
        });
        //console.log(esRepos.body.hits.hits);
        const activeRepos = esRepos.body.hits.hits.map(r => r._source);
        cli_ux_1.default.action.stop(' done');
        if (activeRepos.length === 0) {
            this.error('The script could not find any active repositories. Please use ghRepos and cfRepos first.', { exit: 1 });
        }
        const fetchData = new index_1.default(this.log, gh_token, gh_increment, this.config.configDir, cli_ux_1.default);
        this.log('Starting to grab labels');
        for (let repo of activeRepos) {
            //A - Check if repo index exists, if not create
            const labelsIndex = (indexLabelPrefix + repo.org.login + '_' + repo.name).toLocaleLowerCase();
            const testIndex = await client.indices.exists({ index: labelsIndex });
            if (testIndex.body === false) {
                cli_ux_1.default.action.start('Elasticsearch Index ' + labelsIndex + ' does not exist, creating');
                const mappings = await jsYaml.safeLoad(labels_1.default);
                const settings = await jsYaml.safeLoad(settings_1.default);
                await client.indices.create({ index: labelsIndex, body: { settings, mappings } });
                cli_ux_1.default.action.stop(' created');
            }
            else {
                // If index exists, flush it. It's the only way to ensure records in GitHub and in Elasticsearch remains in sync
                await client.indices.flush({ index: labelsIndex });
            }
            //C - Fetch labels from GitHub into a large array
            cli_ux_1.default.action.start('Grabbing labels for: ' + repo.org.login + '/' + repo.name + ' (will fetch up to ' + repo.labels.totalCount + ' labels)');
            let fetchedLabels = await fetchData.load(repo);
            cli_ux_1.default.action.stop(' done');
            //D - Break down the labels response in multiple batches
            const esPayloadChunked = await chunkArray_1.default(fetchedLabels, 100);
            //E- Push the results back to Elastic Search
            for (const [idx, esPayloadChunk] of esPayloadChunked.entries()) {
                cli_ux_1.default.action.start('Submitting data to ElasticSearch into ' + labelsIndex + ' (' + (idx + 1) + ' / ' + esPayloadChunked.length + ')');
                let formattedData = '';
                for (let rec of esPayloadChunk) {
                    formattedData = formattedData + JSON.stringify({
                        index: {
                            _index: labelsIndex,
                            _id: rec.id
                        }
                    }) + '\n' + JSON.stringify(rec) + '\n';
                }
                await client.bulk({ index: labelsIndex, refresh: 'wait_for', body: formattedData });
                cli_ux_1.default.action.stop(' done');
            }
        }
    }
}
GhLabels.description = 'Fetch labels from GitHub';
GhLabels.examples = [
    '$ github-indexer ghLabels',
];
GhLabels.flags = Object.assign({}, base_1.default.flags, { help: command_1.flags.help({ char: 'h' }) });
exports.default = GhLabels;
